%
% MATLAB_BENs_Positional_Control.m
% for MATLAB R2018a Update 6 (9.4.0.949201) 64bit (win64) Sept 5,2018
%
% V1.0 - Sean Quinn 14 Jan 2019
%
% Use MATLAB to control a Permanent Magnet Motor with Encoder
% with an implementation of BEN's fuzzy control algorithm
%
% Motor + to Matrix EM Control Unit 'A' (or 'U') 
% Motor - to Matrix EM Control Unit 'B' (or 'V')
%
% Note:
% use pathtool or Set Path
% to point to your working directory and include the ~/Primitives directory
%
clear all

global EM_USB;
%
% Find the COM port on your PC where the Matrix EM control unit is plugged
% in and use it here instead of 'COM17'
%
EM_USB = serial('COM17','Baudrate',115200,'Databits',8);
fopen(EM_USB);

%
% Preconditions
%
SetActive(0);         % Motor OFF
pause(0.1);
SetCapacitorBank(0);  % OFF
pause(0.1);
SetDynoLoad(127); % Max Resistance = Minimum Load
pause(0.1);
SetSeriesResistor(0); % 0 Ohms
pause(0.1);
SetMotorType (3);     % Reversible DC (i.e. DC Servomotor)
pause(0.1);
SetMotorDirection(0); % Clockwise
pause(0.1);
ZeroAbsPosition ();  % Zero the Encoder
pause(0.1);
ZeroLoadCell();       % Zero the Load Cell
pause(0.1);

%
% This just tests the comms to the control unit before we start
%
api_version = GetAPIVersion();
disp(['api_version = ' , num2str(api_version)]);

%
% Check the Dynamometer 'motor located' switches
%
switch_state = GetSwitches ();
if switch_state >= 1
   disp('==== PID TEST START ====')
   disp('PID Plate Installed Correctly');
   current_I_U = GetCurrent(0);
   disp(['Current with motor OFF = ' , num2str(current_I_U) , 'A']);
   pause(0.1);
   current_I1 = GetCurrent(4);
   disp(['I1 Current (with Motor OFF) = ' , num2str(current_I1) , 'A']);
   
   Kp       = 10; % K term for Proportional
   Ki       = 10; % K term for Integral
   Kd       = 10; % K term for Differential
   torque   = 0;
   current_error    = 0;
   previous_error   = 0;
   historical_error = 0;

   disp('Move arm to set point now!');
   pause(2);
   setpoint = 0;  % This is the set point
   ZeroAbsPosition ();  % Zero the Encoder
   pause(0.1);   
   
   disp('MOTOR ON');
   SetActive(1); % Motor ON
   pause(0.1); % Wait for the motor to spin up
   %
   % Perform the sampled measurements in as tight a loop as you can.
   %
   for index = 0:1000
      historical_error = previous_error;
      previous_error = current_error;
      current_position = GetAbsPosition ();
      if (current_position > setpoint)
          current_error = (current_position - setpoint);
          SetControlVoltage(previous_error, current_error);
          SetMotorDirection(1);
          SetActive(1); % Motor ON
      elseif (current_position < setpoint)
          current_error = (setpoint - current_position);
          SetControlVoltage(previous_error, current_error);
          SetMotorDirection(0);
          SetActive(1); % Motor ON
      else
          current_error = 0;
          SetVoltage(0);
          SetActive(0); % Motor OFF
      end
      
      %pause(0.1);
   end
   %
   % You can turn off the motor
   % and close the COM port
   %
   disp('MOTOR OFF');
   SetActive(0); % Motor OFF
   pause(0.1);
   fclose(EM_USB)

   disp('==== PID TEST END ======');
else    
   disp('==== PID Plate not fitted in ====');
end

%
% Check for corrupt comms
%
fopen(EM_USB);
api_version_two = GetAPIVersion();
if api_version == api_version_two
   % do nothing
else
    disp('=== CORRUPT COMMS ===');
    disp(['api_version = ' , num2str(api_version_two) , ' it was ' , num2str(api_version) ] );
end
   
fclose(EM_USB) % Close the COM port


%
% SetControlVoltage()
%
% Set the motor control voltage in a Fuzzy / Pseudo Fuzzy way
%
function SetControlVoltage(previous_error, current_error)

% Set motor voltage based on absolute error
voltage_for_motor = current_error * 0.11;

% Adjust for rate of change of error reducing
if(current_error < previous_error)
   voltage_for_motor = current_error * 0.05;
end

% End-stops to avoid over and under driving the motor
if (voltage_for_motor > 14.5)
   voltage_for_motor = 14.5;
elseif (voltage_for_motor < 10)
   voltage_for_motor = 10;
else
   % leave voltage_for_motor unchanged
end

SetVoltage(voltage_for_motor);
 disp(['error= ' , num2str(current_error) , ' V= ' , num2str(voltage_for_motor) ] );
end

%
% End MATLAB_BENs_Positional_Control.m
%


   
   