%
% MATLAB_EM_Plot_AC3PH_Star.m
% for MATLAB R2018a Update 6 (9.4.0.949201) 64bit (win64) Sept 5,2018
%
% V1.0 - Sean Quinn 11 Jan 2019
%
% Plot the 3 Phase AC induction Motor Open Loop Step Response from power up
% Run motor for 2 seconds at 24V AC RMS.
% while sampling RPM, Torque and Current at regular intervals
%
% Plot the AC induction Motor Open Loop Characteristics
%
% Note:
% use pathtool or Set Path
% to point to your working directory and include the ~/Primitives directory
%
clear all

global EM_USB;
%
% Find the COM port on your PC where the Matrix EM control unit is plugged
% in and use it here instead of 'COM17'
%
EM_USB = serial('COM17','Baudrate',115200,'Databits',8);
fopen(EM_USB);

%
% Preconditions
%
SetActive(0);         % Motor OFF
pause(0.1);
SetCapacitorBank(0);  % OFF
pause(0.1);
SetDynoLoad(127); % Max Resistance = Minimum Load
pause(0.1);
SetSeriesResistor(0); % 0 Ohms
pause(0.1);
SetMotorDirection(0); % Clockwise
pause(0.1);
SetMotorType (2);     % AC Three Phase Motor
pause(0.1);

frequency = 50;       % 50Hz
%motor_voltage = 24;   % 24V AC
dyno_load = 20;       % Dynamometer Load Resistance = 20 Ohms
SetFrequency(frequency);
pause(0.1)
%
% This just tests the comms to the control unit before we start
%
api_version = GetAPIVersion();
disp(['api_version = ' , num2str(api_version)]);

%
% Check the Dynamometer 'motor located' switches
%
switch_state = GetSwitches ();
if switch_state >= 1
   disp('==== AC MOTOR TEST START ====')
   disp('Motor Installed Correctly');
   current_I_U = GetCurrent(0);
   disp(['AC Current with motor OFF = ' , num2str(current_I_U) , 'A']);
   pause(0.1);
   %disp(['Setting Motor Voltage to ' , num2str(motor_voltage) , ' Volts']);
   %SetVoltage (motor_voltage);
   pause(0.1);
   disp(['Setting Dyno Load to ' , num2str(dyno_load) , ' Ohms']);
   SetDynoLoad(dyno_load);
   pause(0.1);
   current_I1 = GetCurrent(4);
   disp(['I1 Current (with Motor OFF) = ' , num2str(current_I1) , 'A']);    
   time_interval_seconds = 0.05;
   run_time_seconds = 2;
   loop_count = round((run_time_seconds / time_interval_seconds), 0);
   disp('MOTOR ON');
   SetActive(1); % Motor ON
   pause(0.1);
   %
   % Perform the sampled measurements in as tight a loop as you can.
   %
   for index = 1:loop_count
      time_range(index) = (index * time_interval_seconds);
      encoder_speed_range(index) = GetRPM();
      load_cell_torque_range(index) = GetLoadCell(1);
      current_I_U_range(index) = GetCurrent(0);
      pause(time_interval_seconds);
   end
   %
   % Sampling over, you can turn off the motor
   % and close the COM port
   %
   disp('MOTOR OFF');
   SetActive(0); % Motor OFF
   pause(0.1);
   fclose(EM_USB)

   %
   % Define and move the plots so they are not
   % on top of each other on screen.
   %
   rpm_plot = figure;
   torque_plot = figure;
   current_plot = figure;
   movegui(rpm_plot , 'northwest');
   movegui(torque_plot , 'southwest');
   movegui(current_plot, 'northeast');
   
   %
   % Plot the RPM against Time
   %
   figure(rpm_plot);
   hold on
   p2 = polyfit(time_range, encoder_speed_range, 5); % fit to 5th order Polynomial
   y2 = polyval(p2, time_range);
   plot(time_range, encoder_speed_range, 'r*', time_range, y2, '-r' );
   rpm_polynomial = poly2sym(p2,sym('t'));
   rpm_equation = sprintf('RPM = %s',rpm_polynomial); 
   disp(rpm_equation);
   grid on
   legend('RPM Samples','RPM best fit curve','Location','southeast');
   xlabel('Time, Seconds');
   ylabel('Speed, RPM');
   title(['3 Phase AC Motor RPM from Standing Start @ ', num2str(frequency) , 'Hz']);      
   hold off
   
   %
   % Plot the Torque against Time
   %
   figure(torque_plot);
   hold on
   p3 = polyfit(time_range, load_cell_torque_range, 5); % fit to 5th order Polynomial
   y3 = polyval(p3, time_range);   
   plot(time_range, load_cell_torque_range, 'b+', time_range, y3, '-b' );
   torque_polynomial = poly2sym(p3,sym('t'));
   torque_equation = sprintf('Torque = %s',torque_polynomial); 
   disp(torque_equation);   
   grid on
   legend('Torque Samples', 'Torque best fit curve','Location','southeast');
   xlabel('Time, Seconds');
   ylabel('Torque, mNm');
   title(['3 Phase AC Motor Torque from Standing Start @ ', num2str(frequency) , 'Hz']);      
   hold off
   
   %
   % Plot the Current against Time
   %
   figure(current_plot);
   hold on
   p4 = polyfit(time_range, current_I_U_range, 5); % fit to 5th order Polynomial
   y4 = polyval(p4, time_range);   
   plot(time_range, current_I_U_range, 'kx', time_range, y4, '-k' );
   current_polynomial = poly2sym(p4,sym('t'));
   current_equation = sprintf('Current = %s',current_polynomial); 
   disp(current_equation);    
   grid on
   legend('Current','Current best fit curve');
   xlabel('Time, Seconds');
   ylabel('Current, Amps');
   title(['3 Phase AC Motor Current from Standing Start @ ', num2str(frequency) , 'Hz']);           
   hold off
      
   pause(0.1);
   disp('==== AC MOTOR TEST END ======');
else    
   disp('==== Motor not fitted in Dynomometer ====');
end

%
% Check for corrupt comms
%
fopen(EM_USB);
api_version_two = GetAPIVersion();
if api_version == api_version_two
   % do nothing
else
    disp('=== CORRUPT COMMS ===');
    disp(['api_version = ' , num2str(api_version_two) , ' it was ' , num2str(api_version) ] );
end
   
fclose(EM_USB) % Close the COM port
%
% End of MATLAB_EM_Plot_AC3PH_Star.m
%

