%
% MATLAB_PID_Positional_Control.m
% for MATLAB R2018a Update 6 (9.4.0.949201) 64bit (win64) Sept 5,2018
%
% V1.0 - Sean Quinn 14 Jan 2019
%
% Use MATLAB to control a Permanent Magnet Motor with Encoder
% with a PID control loop
%
% Motor + to Matrix EM Control Unit 'A' (or 'U') 
% Motor - to Matrix EM Control Unit 'B' (or 'V')
%
% Note:
% use pathtool or Set Path
% to point to your working directory and include the ~/Primitives directory
%
clear all

global EM_USB;
%
% Find the COM port on your PC where the Matrix EM control unit is plugged
% in and use it here instead of 'COM17'
%
EM_USB = serial('COM17','Baudrate',115200,'Databits',8);
fopen(EM_USB);

%
% Preconditions
%
SetActive(0);         % Motor OFF
pause(0.1);
SetCapacitorBank(0);  % OFF
pause(0.1);
SetDynoLoad(127); % Max Resistance = Minimum Load
pause(0.1);
SetSeriesResistor(0); % 0 Ohms
pause(0.1);
SetMotorType (3);     % Reversible DC (i.e. DC Servomotor)
pause(0.1);
SetMotorDirection(0); % Clockwise
pause(0.1);
ZeroAbsPosition ();  % Zero the Encoder
pause(0.1);
ZeroLoadCell();       % Zero the Load Cell
pause(0.1);

motor_high_step_voltage = 14;
motor_low_step_voltage  = 11;
motor_voltage_stop      = 0;

%
% This just tests the comms to the control unit before we start
%
api_version = GetAPIVersion();
disp(['api_version = ' , num2str(api_version)]);

%
% Check the Dynamometer 'motor located' switches
%
switch_state = GetSwitches ();
if switch_state >= 1
   disp('==== PID TEST START ====')
   disp('PID Plate Installed Correctly');
   current_I_U = GetCurrent(0);
   disp(['Current with motor OFF = ' , num2str(current_I_U) , 'A']);
   pause(0.1);
   current_I1 = GetCurrent(4);
   disp(['I1 Current (with Motor OFF) = ' , num2str(current_I1) , 'A']);

   %
   % Based on experimental data and sampling the sensor every 0.1 seconds
   %
   % Cohen-Coon   | Tuning 1 |
   % Gain = 0.845 |  0.500   |
   % Kp =   0.008 |  0.010   |
   % Ki =   0.226 |  0.004   |
   % Kd =   0.035 |  0.001   |
   % clamp   n/a  |  0.500   |
   %
   Gain     = 0.8;    % 0.845;
   Kp       = 0.008; % 0.008 K term for Proportional
   Ki       = 0.004; % 0.226; % K term for Integral
   Kd       = 0.001; % 0.035; % K term for Derivative
   integrator_windup_clamp = 50;
      
   current_error         = 0; % steady state error (signed)
   previous_error        = 0;
   derivative_error      = 0; % rate of change of error (signed)
   integral_error        = 0; % running total of current_error (signed)

   control_voltage  = motor_low_step_voltage;
   SetVoltage(control_voltage);
   motor_direction = 0;

   setpoint = 0;  % This is the set point
   ZeroAbsPosition ();  % Zero the Encoder
   pause(0.1);   
   
   disp('MOTOR ON');
   SetActive(1); % Motor ON
   pause(0.1); % Wait for the motor to spin up
   %
   % Perform the sampled measurements in as tight a loop as you can.
   %
   for index = 1:100
      if(index == 10)
         setpoint = 180; % apply a step input to the system
      end
      
      derivative_error = (current_error - previous_error);
      integral_error = integral_error + current_error;
      if(integral_error > integrator_windup_clamp)
          integral_error = integrator_windup_clamp;          
      elseif(integral_error < (integrator_windup_clamp * -1))
          integral_error = (integrator_windup_clamp * -1);
      end
      current_position = GetAbsPosition ();
      previous_error = current_error;
      current_error = (current_position - setpoint); % 10:1 gearbox
      if (current_position > setpoint)
          motor_direction = 1;
          SetMotorDirection(1);
      elseif (current_position < setpoint)
          motor_direction = 0;
          SetMotorDirection(0);
      else
          current_error = 0;
      end
      control_voltage = Gain * ( (Kp * current_error) + (Ki * integral_error) + (Kd * derivative_error));
      if(control_voltage < 0)
          control_voltage = control_voltage * -1;
      end
      
      if(control_voltage > motor_high_step_voltage)
         control_voltage = motor_high_step_voltage;
      else
         control_voltage = control_voltage + motor_low_step_voltage;          
      end
      SetVoltage(control_voltage);
      
      time_range(index) = index;
      if(motor_direction == 0)
         control_voltage_range(index) = (control_voltage) - motor_low_step_voltage;          
      else
         control_voltage_range(index) = (control_voltage * -1) + motor_low_step_voltage;           
      end
      current_error_range(index) = current_error;
      integral_error_range(index) = integral_error;
      derivative_error_range(index) = derivative_error;
      
      disp(['current_error = ', num2str(current_error), ' integral_error = ', num2str(integral_error), ' derivative_error = ', num2str(derivative_error),  ' control_voltage = ' , num2str(control_voltage)]);
      
      pause(0.1);
   end
   %
   % You can turn off the motor
   % and close the COM port
   %
   disp('MOTOR OFF');
   SetActive(0); % Motor OFF
   pause(0.1);
   fclose(EM_USB)

   %
   % Define and move the plots so they are not
   % on top of each other on screen.
   %
   error_plot = figure;
   control_plot = figure;
   movegui(error_plot , 'northwest');
   movegui(control_plot , 'northeast');
   %
   % Plot of current_error against Time
   %
   figure(error_plot);
   hold on
   plot(time_range, current_error_range, 'r-+', time_range, integral_error_range, 'g-x', time_range, derivative_error_range, 'b-*');
   grid on
   legend('current error','Location','northeast','integral error','Location','northeast','derivative error','Location','northeast' );
   xlabel('Time, @ sample intervals');
   ylabel('error in degrees from setpoint');
   title('error from setpoint');      
   hold off   
   %
   % Plot of control_voltage against Time
   %
   figure(control_plot);
   hold on
   plot(time_range, control_voltage_range, 'r-x');
   grid on
   legend('control voltage','Location','northeast');
   xlabel('Time, @ sample intervals');
   ylabel('error in degrees from setpoint');
   title(['Setpoint Control with voltage Low to High Limits = ', num2str(motor_low_step_voltage), 'V to ', num2str(motor_high_step_voltage), 'V ' ]);      
   hold off     
   disp('==== PID TEST END ======');
else    
   disp('==== PID Plate not fitted in ====');
end

%
% Check for corrupt comms
%
fopen(EM_USB);
api_version_two = GetAPIVersion();
if api_version == api_version_two
   % do nothing
else
    disp('=== CORRUPT COMMS ===');
    disp(['api_version = ' , num2str(api_version_two) , ' it was ' , num2str(api_version) ] );
end
   
fclose(EM_USB) % Close the COM port
%
% MATLAB_PID_Positional_Control.m
%


   