%
% MATLAB_Plot_DC_PM_Step.m
% for MATLAB R2018a Update 6 (9.4.0.949201) 64bit (win64) Sept 5,2018
%
% V1.0 - Sean Quinn 22 Jan 2019
%
% Plot the DC Permanent Magnet Motor Open Loop Step Response
% Runs motor for 1 second at 10V
% then
% increases voltage to 20V
% while sampling RPM, Torque and Current at regular intervals
%
% Note:
% use pathtool or Set Path
% to point to your working directory and include the ~/Primitives directory
%
clear all
clc
global EM_USB;
%
% Find the COM port on your PC where the Matrix EM control unit is plugged
% in and use it here instead of 'COM17'
%
EM_USB = serial('COM17','Baudrate',115200,'Databits',8);
fopen(EM_USB);

SetActive(0);                    % Motor OFF
SetMotorType (0);                % DC Motor
motor_voltage_before_step = 10;  % 10V DC
motor_voltage_after_step  = 20;  % 20V DC
dyno_load                 = 40;  % Apply a Dynamometer load
SetDynoLoad(dyno_load);      % Max Resistance (255) = Minimum Load

%
% Test the comms to the control unit before we start
%
api_version = GetAPIVersion();
disp(['api_version = ' , num2str(api_version)]);

%
% Check that the motor is connected to the Dynamometer
%
switch_state = GetSwitches();
if switch_state >= 1
   disp('==== Motor Installed Correctly ====');
   current_I_DC = GetCurrent(3);
   disp(['DC Current with motor OFF = ' , num2str(current_I_DC) , 'A']);
   disp(['Setting Motor Voltage to ' , num2str(motor_voltage_before_step) , ' Volts']);
   SetVoltage (motor_voltage_before_step);    
   time_interval_seconds = 0.1;
   run_time_seconds = 2;
   loop_count = round((run_time_seconds / time_interval_seconds), 0);
   step_index = loop_count / 2;
   disp('MOTOR ON');
   SetActive(1); % Motor ON
   %
   % Perform the sampled measurements in a tight loop.
   %
   for index = 1:loop_count
      time_range(index) = (index * time_interval_seconds);
      encoder_speed_range(index) = GetRPM();
      load_cell_torque_range(index) = GetLoadCell(1);
      current_I_DC_range(index) = GetCurrent(4); % Should be 3 for I_DC
      if (index == step_index)
         SetVoltage (motor_voltage_after_step);
      end
      pause(time_interval_seconds);
   end
   %
   % Sampling over, turn off the motor
   % and close the COM port
   %
   disp('MOTOR OFF');
   SetActive(0); % Motor OFF
   pause(0.1);
   fclose(EM_USB)
   
   %
   % Define and move the plots so they are not
   % on top of each other on screen.
   %
   rpm_plot = figure;
   torque_plot = figure;
   current_plot = figure;
   movegui(rpm_plot , 'northwest');
   movegui(torque_plot , 'southwest');
   movegui(current_plot, 'northeast');
   
   %
   % Plot the RPM against Time
   %
   figure(rpm_plot);
   hold on
   plot(time_range, encoder_speed_range, 'r-*');
   grid on
   legend('RPM','Location','southeast');
   xlabel('Time, Seconds');
   ylabel('Speed, RPM');
   title(['PM DC Motor RPM step response from ', num2str(motor_voltage_before_step) , 'V to ' , num2str(motor_voltage_after_step) , 'V' ]);      
   hold off
   
   %
   % Plot Torque against Time
   %
   figure(torque_plot);
   hold on
   plot(time_range, load_cell_torque_range, 'b-+');
   grid on
   legend('Load','Location','southeast');
   xlabel('Time, Seconds');
   ylabel('Torque, mNm');
   title(['PM DC Motor Torque step response from ', num2str(motor_voltage_before_step) , 'V to ' , num2str(motor_voltage_after_step) , 'V' ]);           
   hold off
   
   %
   % Plot the Current against Time
   %
   figure(current_plot);
   hold on
   plot(time_range, current_I_DC_range, 'k-x');
   grid on
   legend('Current');
   xlabel('Time, Seconds');
   ylabel('Current, Amps');
   title(['PM DC Motor Current step response from ', num2str(motor_voltage_before_step) , 'V to ' , num2str(motor_voltage_after_step) , 'V' ]);             
   hold off
   
   pause(0.1);
   disp('==== Motor Test Finished ====');
else    
   disp('==== Motor not fitted in Dynomometer ====');
end

%
% End of MATLAB_Plot_DC_PM_Step.m
%


