%
% MATLAB_Positional_Control_open_loop_step.m
% for MATLAB R2018a Update 6 (9.4.0.949201) 64bit (win64) Sept 5,2018
%
% V1.0 - Sean Quinn 16 Jan 2019
%
% Use MATLAB to determine the open loop step response of
% a PMDC motor with encoder feedback.
%
% Motor + to Matrix EM Control Unit 'A' (or 'U') 
% Motor - to Matrix EM Control Unit 'B' (or 'V')
%
% Note:
% use pathtool or Set Path
% to point to your working directory and include the ~/Primitives directory
%
clear all
clc

disp('MATLAB_Positional_Control_open_loop_step.m');

global EM_USB;
%
% Find the COM port on your PC where the Matrix EM control unit is plugged
% in and use it here instead of 'COM17'
%
EM_USB = serial('COM17','Baudrate',115200,'Databits',8);
fopen(EM_USB);

%
% Preconditions
%
SetActive(0);         % Motor OFF
pause(0.1);
SetCapacitorBank(0);  % OFF
pause(0.1);
SetDynoLoad(127); % Max Resistance = Minimum Load
pause(0.1);
SetSeriesResistor(0); % 0 Ohms
pause(0.1);
SetMotorType (3);     % Reversible DC (i.e. DC Servomotor)
pause(0.1);
SetMotorDirection(0); % Clockwise
pause(0.1);
ZeroAbsPosition ();  % Zero the Encoder
pause(0.1);
ZeroLoadCell();       % Zero the Load Cell
pause(0.1);

motor_high_step_voltage = 18;
motor_nominal_voltage   = 16;
motor_low_step_voltage  = 14;

%
% This just tests the comms to the control unit before we start
%
api_version = GetAPIVersion();
disp(['api_version = ' , num2str(api_version)]);

%
% Check the Dynamometer 'motor located' switches
%
switch_state = GetSwitches ();
if switch_state >= 1
   disp('==== Positional Step TEST START ====')
   disp('Inverted Pendulum Plate Installed Correctly');
   current_I_U = GetCurrent(0);
   disp(['Current with motor OFF = ' , num2str(current_I_U) , 'A']);
   pause(0.1);
   current_I1 = GetCurrent(4);
   disp(['I1 Current (with Motor OFF) = ' , num2str(current_I1) , 'A']);
   
   ZeroAbsPosition ();  % Zero the Encoder
   pause(0.1);   
   time_interval_seconds = 0.1;
   run_time_seconds = 16;
   loop_count = round((run_time_seconds / time_interval_seconds), 0);
   loop_state_number = 16; % number of loop_states below
   loop_state = 2; % 1 = fwd nominal, 2 = fwd Low step, 3 = fwd high step
                   % 4 = fwd nominal, 5 = fwd high step, 6 = fwd low step
                   % 7 = fwd nominal, 8 = stop,
                   % 9 = rev_nominal, 10 = rev_low_step, 11 = rev_high_step
                   % 12 = rev_nominal, 13 = rev_high_step,
                   % 14 = rev_low_step, 15 = rev_nominal, 16 = stop
   step_index = loop_count / loop_state_number;
   SetMotorDirection(1);
   SetVoltage(motor_nominal_voltage);
   disp('MOTOR ON');
   disp('nominal');   
   SetActive(1); % Motor ON

   %
   % Perform the sampled measurements in as tight a loop as you can.
   %
   for index = 1:loop_count
      time_range(index) = (index * time_interval_seconds);
      encoder_speed_range(index) = GetRPM();

      if (index == step_index)
         step_index = step_index + loop_count / loop_state_number;
         switch(loop_state)
             case 1
                SetVoltage(motor_low_step_voltage);
                disp('low step');
                loop_state = 2;
             case 2
                SetVoltage(motor_low_step_voltage);
                disp('low step');
                loop_state = 3;
             case 3
                SetVoltage(motor_high_step_voltage);
                disp('high step');
                loop_state = 4;
             case 4
                SetVoltage(motor_nominal_voltage);
                disp('nominal');
                loop_state = 5;
             case 5
                SetVoltage(motor_high_step_voltage);
                disp('high step');
                loop_state = 6;
             case 6
                SetVoltage(motor_low_step_voltage);
                disp('low step');
                loop_state = 7;
             case 7
                SetVoltage(motor_nominal_voltage);
                disp('nominal');
                loop_state = 8;
             case 8
                SetVoltage(0);
                disp('stop');
                SetMotorDirection(0); % Reverse direction
                loop_state = 9;
             case 9
                SetVoltage(motor_nominal_voltage);
                disp('nominal');
                loop_state = 10;                 
             case 10
                SetVoltage(motor_low_step_voltage);
                disp('low step');
                loop_state = 11;                 
             case 11
                SetVoltage(motor_high_step_voltage);
                disp('high step');
                loop_state = 12;                 
             case 12
                SetVoltage(motor_nominal_voltage);
                disp('nominal');
                loop_state = 13;                   
             case 13
                SetVoltage(motor_high_step_voltage);
                disp('high step');
                loop_state = 14;                 
             case 14
                SetVoltage(motor_low_step_voltage);
                disp('low step');
                loop_state = 15;                 
             case 15
                SetVoltage(motor_nominal_voltage);
                disp('nominal');
                loop_state = 16;                 
             case 16
                SetVoltage(0);
                disp('stop');                 
             otherwise % should never happen
                disp('loop_state otherwise ERROR');
                SetVoltage (0);                  
         end
      end
      pause(time_interval_seconds);
   end 
   %
   % You can turn off the motor
   % and close the COM port
   %
   disp('MOTOR OFF');
   SetActive(0); % Motor OFF
   pause(0.1);
   fclose(EM_USB)

   %
   % Define and move the plots so they are not
   % on top of each other on screen.
   %
   rpm_plot = figure;
   movegui(rpm_plot , 'northwest');
   
   %
   % Plot the RPM against Time
   %
   figure(rpm_plot);
   hold on
   plot(time_range, encoder_speed_range, 'r-*');
   grid on
   legend('RPM','Location','northeast');
   xlabel('Time, Seconds');
   ylabel('Speed, RPM');
   title(['Voltages (low step, nominal, high step) = ', num2str(motor_low_step_voltage), 'V ,', num2str(motor_nominal_voltage), 'V, ', num2str(motor_high_step_voltage), 'V with 0->7 seconds forward, 8->16 seconds reverse' ]);      
   hold off
   
   
   
   
   disp('==== Positional Step TEST END ====');
else    
   disp('==== Inverted Pendulum Plate not fitted in ====');
end

%
% Check for corrupt comms
%
fopen(EM_USB);
api_version_two = GetAPIVersion();
if api_version == api_version_two
   % do nothing
else
    disp('=== CORRUPT COMMS ===');
    disp(['api_version = ' , num2str(api_version_two) , ' it was ' , num2str(api_version) ] );
end
   
fclose(EM_USB) % Close the COM port

%
% End MATLAB_Positional_Control_open_loop_step.m
%


   
   