%
% MATLAB_Positional_Control_open_loop_step_ANGLE.m
% for MATLAB R2018a Update 6 (9.4.0.949201) 64bit (win64) Sept 5,2018
%
% V1.0 - Sean Quinn 16 Jan 2019
%
% Use MATLAB to determine the open loop step response of
% a PMDC motor with encoder feedback.
%
% Motor + to Matrix EM Control Unit 'A' (or 'U') 
% Motor - to Matrix EM Control Unit 'B' (or 'V')
%
% Note:
% use pathtool or Set Path
% to point to your working directory and include the ~/Primitives directory
%
clear all
clc

disp('MATLAB_Positional_Control_open_loop_step_ANGLE.m');

global FA_USB;
%
% Find the COM port on your PC where the Matrix EM control unit is plugged
% in and use it here instead of 'COM17'
%
FA_USB = serial('COM17','Baudrate',115200,'Databits',8);
fopen(FA_USB);

%
% Preconditions
%
SetActive(0);         % Motor OFF
pause(0.1);
SetCapacitorBank(0);  % OFF
pause(0.1);
SetResistorBank(127); % Max Resistance = Minimum Load
pause(0.1);
SetSeriesResistor(0); % 0 Ohms
pause(0.1);
SetMotorType (3);     % Reversible DC (i.e. DC Servomotor)
pause(0.1);
SetMotorDirection(0); % Clockwise
pause(0.1);
ResetAbsPosition ();  % Zero the Encoder
pause(0.1);
ZeroLoadCell();       % Zero the Load Cell
pause(0.1);

motor_high_step_voltage = 15;
motor_low_step_voltage  = 10;
motor_voltage_stop      = 0;
min_angle_achieved      = 0;
max_angle_achieved      = 0;

%
% This just tests the comms to the control unit before we start
%
api_version = GetAPIVersion();
disp(['api_version = ' , num2str(api_version)]);

%
% Check the Dynamometer 'motor located' switches
%
switch_state = GetSwitches ();
if switch_state >= 1
   disp('==== Positional Step TEST START ====')
   disp('Inverted Pendulum Plate Installed Correctly');
   current_I_U = GetCurrent(0);
   disp(['Current with motor OFF = ' , num2str(current_I_U) , 'A']);
   pause(0.1);
   current_I1 = GetCurrent(4);
   disp(['I1 Current (with Motor OFF) = ' , num2str(current_I1) , 'A']);
   
   ResetAbsPosition ();  % Zero the Encoder
   pause(0.1);   
   time_interval_seconds = 0.1;
   run_time_seconds = 4;
   loop_count = round((run_time_seconds / time_interval_seconds), 0);
   loop_state_number = 5; % number of loop_states below
   loop_state = 2; % 1 = fwd nominal, 2 = fwd Low step, 3 = fwd high step
                   % 4 = fwd nominal, 5 = fwd high step
   step_index = loop_count / loop_state_number;
   SetMotorDirection(0);
   motor_voltage = motor_voltage_stop;
   SetVoltage(motor_voltage);
   disp('MOTOR ON');
   disp('Stop');   
   SetActive(1); % Motor ON

   %
   % Perform the sampled measurements in as tight a loop as you can.
   %
   for index = 1:loop_count
      time_range(index) = (index * time_interval_seconds);
      absolute_position_range(index) = GetAbsPosition();
      if(absolute_position_range(index) > max_angle_achieved)
          max_angle_achieved = absolute_position_range(index);
      end
      
      if((motor_voltage - motor_low_step_voltage) < 0)
          step_input_range(index) = 0;
      else
          step_input_range(index) = (motor_voltage - motor_low_step_voltage) * 1000;
      end

      if (index == step_index)
         step_index = step_index + loop_count / loop_state_number;
         switch(loop_state)
             case 1
                motor_voltage = motor_low_step_voltage;
                SetVoltage(motor_voltage);
                disp('Low Step');
                loop_state = 2;
             case 2
                motor_voltage = motor_high_step_voltage;
                SetVoltage(motor_voltage);
                disp('High Step');
                loop_state = 3;
             case 3
                motor_voltage = motor_low_step_voltage;
                SetVoltage(motor_voltage);
                disp('Low Step');
                SetMotorDirection(1); % Reverse direction                
                loop_state = 4;
             case 4
                motor_voltage = motor_high_step_voltage;
                SetVoltage(motor_voltage);
                disp('High Step');
                loop_state = 5;
             case 5
                motor_voltage = motor_low_step_voltage;
                SetVoltage(motor_voltage);
                disp('Low Step');                 
             otherwise % should never happen
                disp('loop_state otherwise ERROR');
                motor_voltage = motor_voltage_stop;
                SetVoltage (motor_voltage);                  
         end
      end
      pause(time_interval_seconds);
   end 
   %
   % You can turn off the motor
   % and close the COM port
   %
   disp('MOTOR OFF');
   SetActive(0); % Motor OFF
   pause(0.1);
   fclose(FA_USB)

   %
   % Define and move the plots so they are not
   % on top of each other on screen.
   %
   rpm_plot = figure;
   movegui(rpm_plot , 'northwest');
   
   %
   % Plot the RPM against Time
   %
   figure(rpm_plot);
   hold on
   plot(time_range, absolute_position_range, 'r-*', time_range, step_input_range, 'b-+' );
   grid on
   legend('Absolute Position','Location','northeast', 'Step Input','Location','northeast');
   xlabel('Time, Seconds');
   ylabel('Absolute Position, Degrees');
   title(['Voltage Low to High Step = ', num2str(motor_low_step_voltage), 'V to ', num2str(motor_high_step_voltage), 'V ' ]);      
   hold off
   %
   % Show Step Input to Angle Output Response
   %
   disp(['min_angle_achieved = ',num2str(min_angle_achieved),' max_angle_achieved = ',num2str(max_angle_achieved) ]);
   disp(['time_range = [ ',num2str(time_range), ' ];']);
   disp(['step_input_range = [ ',num2str(step_input_range), ' ];']);   
   disp(['absolute_position_range = [ ',num2str(absolute_position_range), ' ];']);
   
   disp('==== Positional Step TEST END ====');
else    
   disp('==== Inverted Pendulum Plate not fitted in ====');
end

%
% Check for corrupt comms
%
fopen(FA_USB);
api_version_two = GetAPIVersion();
if api_version == api_version_two
   % do nothing
else
    disp('=== CORRUPT COMMS ===');
    disp(['api_version = ' , num2str(api_version_two) , ' it was ' , num2str(api_version) ] );
end
   
fclose(FA_USB) % Close the COM port

%
% End MATLAB_Positional_Control_open_loop_step_ANGLE.m
%


   
   